@@ -18,6 +18,8 @@
 
 #include "src/core/lib/slice/slice.h"
 
+#include <grpc/slice.h>
+#include <grpc/support/port_platform.h>
 #include <inttypes.h>
 #include <string.h>
 
@@ -31,14 +33,10 @@
 #include "absl/log/log.h"
 #include "absl/strings/string_view.h"
 #include "gtest/gtest.h"
-
-#include <grpc/slice.h>
-#include <grpc/support/port_platform.h>
-
-#include "src/core/lib/gprpp/memory.h"
-#include "src/core/lib/gprpp/no_destruct.h"
 #include "src/core/lib/slice/slice_internal.h"
 #include "src/core/lib/slice/slice_refcount.h"
+#include "src/core/util/memory.h"
+#include "src/core/util/no_destruct.h"
 #include "test/core/test_util/build.h"
 
 TEST(GrpcSliceTest, MallocReturnsSomethingSensible) {
